@@ -1,116 +1,80 @@
-from bw2data import get_activity, Method
-from bw2calc import LCA
 from collections import defaultdict
+from warnings import warn
+
+from bw2calc import LCA
+from bw2data import Method, get_activity, Database
 
 
 def traverse_tagged_databases(
-    functional_unit, method, label="tag", default_tag="other", secondary_tags=[]
+    functional_unit, method, label="tag", default_tag="other", secondary_tags=[], fg_databases=None
 ):
 
-    """Traverse a functional unit throughout its foreground database(s), and
-
-    group impacts by tag label.
-
+    """Traverse a functional unit throughout its foreground database(s) or the 
+    listed databses in fg_databses, and group impacts by tag label.
 
     Contribution analysis work by linking impacts to individual activities.
-
     However, you also might want to group impacts in other ways. For example,
-
     give individual biosphere exchanges their own grouping, or aggregate two
-
     activities together.
 
-
     Consider this example system, where the letters are the tag labels, and the
-
     numbers are exchange amounts. The functional unit is one unit of the tree
-
     root.
 
-
     .. image:: images/tagged-traversal.png
-
        :alt: Example tagged supply chain
 
-
     In this supply chain, tags are applied to activities and biosphere exchanges.
-
     If a biosphere exchange is not tagged, it inherits the tag of its producing
-
     activity. Similarly, links to other databases are assessed with the usual
-
     LCA machinery, and the total LCA score is tagged according to its consuming
-
     activity. If an activity does not have a tag, a default tag is applied.
 
-
     We can change our visualization to show the use of the default tags:
 
-
     .. image:: images/tagged-traversal-2.png
-
        :alt: Example tagged supply chain
 
-
     And then we can manually calculate the tagged impacts. Normally we would
-
     need to know the actual biosphere flows and their respective
-
     characterization factors (CF), but in this example we assume that each
-
     CF is one. Our result, group by tags, would therefore be:
 
-
         * **A**: :math:`6 + 27 = 33`
-
         * **B**: :math:`30 + 44 = 74`
-
         * **C**: :math:`5 + 16 + 48 = 69`
-
         * **D**: :math:`14`
 
-
     This function will only traverse the foreground database, i.e. the
-
     database of the functional unit activity. A functional unit can have
-
     multiple starting nodes; in this case, all foreground databases are
-
     traversed.
 
-
     Input arguments:
 
         * ``functional_unit``: A functional unit dictionary, e.g. ``{("foo", "bar"): 42}``.
-
         * ``method``: A method name, e.g. ``("foo", "bar")``
-
         * ``label``: The label of the tag classifier. Default is ``"tag"``
-
         * ``default_tag``: The tag classifier to use if none was given. Default is ``"other"``
-
         * ``secondary_tags``: List of tuples in the format (secondary_label, secondary_default_tag). Default is empty list.
-
+        * ``fg_databases``: a list of foreground databases to be traversed, e.g. ['foreground', 'biomass', 'machinery']
+                            It's not recommended to include all databases of a project in the list to be traversed, especially not ecoinvent itself
 
     Returns:
 
-
         Aggregated tags dictionary from ``aggregate_tagged_graph``, and tagged supply chain graph from ``recurse_tagged_database``.
 
-
     """
 
     lca = LCA(functional_unit, method)
-
-    lca.lci(factorize=True)
-
+    lca.lci()
     lca.lcia()
 
     method_dict = {o[0]: o[1] for o in Method(method).load()}
 
     graph = [
         recurse_tagged_database(
-            key, amount, method_dict, lca, label, default_tag, secondary_tags
+            key, amount, method_dict, lca, label, default_tag, secondary_tags, fg_databases
         )
         for key, amount in functional_unit.items()
     ]
@@ -144,7 +108,7 @@ def aggregate_tagged_graph(graph):
 
 
 def recurse_tagged_database(
-    activity, amount, method_dict, lca, label, default_tag, secondary_tags=[]
+    activity, amount, method_dict, lca, label, default_tag, secondary_tags=[], fg_databases=None, warned=False
 ):
 
     """Traverse a foreground database and assess activities and biosphere flows by tags.
@@ -152,90 +116,75 @@ def recurse_tagged_database(
 
     Input arguments:
 
-
         * ``activity``: Activity tuple or object
-
         * ``amount``: float
-
         * ``method_dict``: Dictionary of biosphere flow tuples to CFs, e.g. ``{("biosphere", "foo"): 3}``
-
         * ``lca``: An ``LCA`` object that is already initialized, i.e. has already calculated LCI and LCIA with same method as in ``method_dict``
-
         * ``label``: string
-
         * ``default_tag``: string
-
         * ``secondary_tags``: List of tuples in the format (secondary_label, secondary_default_tag). Default is empty list.
-
+        
+        * ``fg_databases``: a list of foreground databases to be traversed, e.g. ['foreground', 'biomass', 'machinery']
+                            It's not recommended to include all databases of a project in the list to be traversed, especially not ecoinvent itself
 
     Returns:
 
-
     .. code-block:: python
 
-
         {
-
             'activity': activity object,
-
             'amount': float,
-
             'tag': string,
-
             'secondary_tags': [list of strings],
-
             'impact': float (impact of inputs from outside foreground database),
-
             'biosphere': [{
-
                 'amount': float,
-
                 'impact': float,
-
                 'tag': string,
-
                 'secondary_tags': [list of strings]
-
             }],
-
             'technosphere': [this data structure]
-
         }
 
-
     """
-
     if isinstance(activity, tuple):
-
         activity = get_activity(activity)
 
+    MESSAGE = """Given databases include many activities, and traversal may be slow.
+Consider using `GraphTraversalLCA` from `bw2calc` instead."""
+
+    if fg_databases is None:  # then set the list equal to the database of the functional unit
+        fg_databases = [activity['database']]  # list, single item
+    elif not warned and sum(len(Database(name)) for name in fg_databases) > 2500:
+        warn(MESSAGE)
+        warned = True
+
     inputs = list(activity.technosphere())
-
     production = list(activity.production())
-    if len(production) == 1:
-        scale = production[0]["amount"]
-    elif not production:
-        # Assume production amount of 1
-        scale = 1
-    else:
-        raise ValueError("Can't scale by production exchange")
 
-    inside = [exc for exc in inputs if exc["input"][0] == activity["database"]]
+    if not production:
+        scale = 1
+    elif len(production) > 1:
+        warn("Hit multiple production exchanges; aborting in this branch")
+        return
+    else:
+        scale = production[0]["amount"]
+        for other in activity.technosphere():
+            if other.input == production[0].input:
+                scale -= other["amount"]
+
+    inside = [exc for exc in inputs if exc.input["database"] in fg_databases]
 
     outside = {
-        exc["input"]: exc["amount"] / scale * amount
+        exc.input.id: exc["amount"] / scale * amount
         for exc in inputs
-        if exc["input"][0] != activity["database"]
+        if exc["input"][0] not in fg_databases
     }
 
     if outside:
-
         lca.redo_lcia(outside)
-
         outside_score = lca.score
-
     else:
-
         outside_score = 0
 
     return {
@@ -246,6 +195,7 @@ def recurse_tagged_database(
         "impact": outside_score,
         "biosphere": [
             {
+                "activity": exc.input,
                 "amount": exc["amount"] / scale * amount,
                 "impact": exc["amount"]
                 / scale
@@ -260,13 +210,15 @@ def recurse_tagged_database(
         ],
         "technosphere": [
             recurse_tagged_database(
-                exc.input,
-                exc["amount"] / scale * amount,
-                method_dict,
-                lca,
-                label,
-                default_tag,
-                secondary_tags,
+                activity=exc.input,
+                amount=exc["amount"] / scale * amount,
+                method_dict=method_dict,
+                lca=lca,
+                label=label,
+                default_tag=default_tag,
+                secondary_tags=secondary_tags,
+                fg_databases=fg_databases,
+                warned=warned,
             )
             for exc in inside
         ],
@@ -394,9 +346,9 @@ def multi_recurse_tagged_database(
         activity = get_activity(activity)
 
     inputs = list(activity.technosphere())
-    inside = [exc for exc in inputs if exc["input"][0] == activity["database"]]
+    inside = [exc for exc in inputs if exc.input["database"] == activity["database"]]
     outside = {
-        exc["input"]: exc["amount"] * amount
+        exc.input.id: exc["amount"] * amount
         for exc in inputs
         if exc["input"][0] != activity["database"]
     }
@@ -418,6 +370,7 @@ def multi_recurse_tagged_database(
         "impact": outside_scores,
         "biosphere": [
             {
+                "activity": exc.input,
                 "amount": exc["amount"] * amount,
                 "impact": [
                     exc["amount"] * amount * method_dict.get(exc["input"], 0)
